"use strict";
var sinon = require("sinon");
var chai = require("chai");
var validator_middleware_1 = require("./validator.middleware");
var Joi = require("joi");
var _ = require("lodash");
var errorbuilder_1 = require("./errorbuilder");
describe('Validator', function () {
    var _classUnderTest;
    var assert = sinon.assert;
    var expect = chai.expect;
    var _req, _res, _next;
    describe('bindModel', function () {
        beforeEach(function () {
            _req = {
                body: {
                    bodyProp: 'one'
                },
                params: {
                    paramProp: 'two'
                },
                query: {
                    queryProp: 'three'
                }
            };
            var sspy = sinon.spy();
            _res = {
                status: sinon.stub().returns({ send: sspy }),
                send: sspy
            };
            _next = sinon.spy();
            _classUnderTest = new validator_middleware_1.ValidatorMiddleware();
        });
        it('should place param and body properties on _model', function (done) {
            var expected = _.extend({}, _req.body, _req.param, _req.query);
            _classUnderTest.bindModel(_req, _res, _next);
            expect(_req._model).contain(expected);
            done();
        });
        it('should call next', function (done) {
            _classUnderTest.bindModel(_req, _res, _next);
            assert.calledOnce(_next);
            assert.calledWithExactly(_next);
            done();
        });
    });
    describe('validateModel', function () {
        var _translatedModel, _translatedErrors;
        beforeEach(function () {
            _translatedModel = { x: 1, y: 'q' };
            _translatedErrors = { field: 'test', messages: [{ err: 1, er2: 'abc' }] };
            Joi.validate = sinon
                .stub()
                .returns({ error: undefined, value: _translatedModel });
            errorbuilder_1.errorBuilder.buildErrors = sinon.stub().returns(_translatedErrors);
            _req = {
                _model: { m: 1 },
                _schema: { s: 2 }
            };
            var sspy = sinon.spy();
            _res = {
                status: sinon.stub().returns({ send: sspy }),
                send: sspy
            };
            _next = sinon.spy();
            _classUnderTest = new validator_middleware_1.ValidatorMiddleware();
        });
        it('should call validate on Joi with the model, schema, and default options', function (done) {
            _classUnderTest.validateModel(_req, _res, _next);
            assert.calledOnce(Joi.validate);
            assert.calledWith(Joi.validate, { m: 1 }, { s: 2 }, {
                context: _req,
                allowUnknown: true,
                abortEarly: false
            });
            done();
        });
        it('should set model and call next when no errors are returned from Joi', function (done) {
            _classUnderTest.validateModel(_req, _res, _next);
            expect(_req._model).equals(_translatedModel);
            assert.calledOnce(_next);
            assert.calledWith(_next);
            done();
        });
        it('should set model and call next when no error details returned from Joi', function (done) {
            Joi.validate = sinon
                .stub()
                .returns({ error: { details: [] }, value: _translatedModel });
            _classUnderTest.validateModel(_req, _res, _next);
            expect(_req._model).equals(_translatedModel);
            assert.calledOnce(_next);
            assert.calledWith(_next);
            done();
        });
        it('should call buildErrors with joi errors when joi returns errors', function (done) {
            var localError = { details: [{ err: 1, er2: 'abc' }] };
            Joi.validate = sinon
                .stub()
                .returns({ error: localError, value: _translatedModel });
            _classUnderTest.validateModel(_req, _res, _next);
            assert.calledWith(errorbuilder_1.errorBuilder.buildErrors, localError);
            done();
        });
        it('should return 400 with built errors when joi returns errors', function (done) {
            var localError = { details: [{ err: 1, er2: 'abc' }] };
            Joi.validate = sinon
                .stub()
                .returns({ error: localError, value: _translatedModel });
            _classUnderTest.validateModel(_req, _res, _next);
            assert.calledOnce(_res.status);
            assert.calledWith(_res.status, 400);
            assert.calledOnce(_res.send);
            assert.calledWith(_res.send, { message: 'Bad Request', errors: _translatedErrors });
            done();
        });
    });
});
//# sourceMappingURL=validator.middleware.spec.js.map